@@ -20,6 +20,8 @@ import shutil
 import tempfile
 import traceback
 import zipfile
+import git
+import pkg_resources
 
 from concurrent.futures.thread import ThreadPoolExecutor
 from io import BytesIO, StringIO
@@ -119,7 +121,7 @@ def login():
     if not verify_password(username, password):
         logging.warning(f"LOGIN FAILED: {username} {password}")
         return make_response(jsonify({
-            'error': "Login failed: wrong user or password"
+            'error': "Login failed: wrong username or password"
         }), 401)
     else:
         user = curr_app.users.get(username)
@@ -267,6 +269,22 @@ def get_workspace_info(workspace_id):
     return jsonify(res)
 
 
+@main_blueprint.route("/workspace/<workspace_id>/load_dataset", methods=['POST'])
+@login_if_required
+@validate_workspace_id
+def load_dataset(workspace_id):
+    """
+    Initial load of large datasets can take a while. The UI can call this endpoint to start a background loading of the
+    dataset to the memory immediately after the workspace was selected. This will save time in subsequent calls to
+    endpoints which require the dataset.
+
+    :param workspace_id:
+    """
+    executor.submit(curr_app.orchestrator_api.preload_dataset, workspace_id)
+    return jsonify({"success": True})
+
+
+
 """
 Category endpoints. A category is defined in the context of a particular workspace. As a user works on the system, all
 the labels, classification models etc. are associated with a specific category.
@@ -314,6 +332,7 @@ def get_all_categories(workspace_id):
                       for id, category in categories.items()]
 
     res = {'categories': category_dicts}
+
     return jsonify(res)
 
 
@@ -393,8 +412,7 @@ def get_all_document_uris(workspace_id):
     """
 
     doc_uris = curr_app.orchestrator_api.get_all_document_uris(workspace_id)
-    res = {"documents":
-               [{"document_id": uri} for uri in doc_uris]}  # TODO change document_id to document_uri in the ui
+    res = {"documents": [{"document_id": uri} for uri in doc_uris]}
     return jsonify(res)
 
 
@@ -420,10 +438,11 @@ def get_document_elements(workspace_id, document_uri):
     category_id = request.args.get('category_id')
     if category_id is not None:
         category_id = int(category_id)
+    hit_count = len(elements)
+    elements = elements[start_idx: start_idx + size]
     elements_transformed = elements_back_to_front(workspace_id, elements, category_id)
-    elements_transformed = elements_transformed[start_idx: start_idx + size]
 
-    res = {'elements': elements_transformed, 'hit_count': len(elements)}
+    res = {'elements': elements_transformed, 'hit_count': hit_count}
     return jsonify(res)
 
 
@@ -459,8 +478,8 @@ def get_document_positive_predictions(workspace_id, document_uri):
         positive_predicted_elements = [element for element, prediction in zip(elements, predictions)
                                        if prediction == LABEL_POSITIVE]
 
+        positive_predicted_elements = positive_predicted_elements[start_idx: start_idx + size]
         elements_transformed = elements_back_to_front(workspace_id, positive_predicted_elements, category_id)
-        elements_transformed = elements_transformed[start_idx: start_idx + size]
     res = {'elements': elements_transformed}
     return jsonify(res)
 
@@ -473,6 +492,8 @@ def get_positive_predictions(workspace_id):
     """
     Get elements in the given workspace that received a positive prediction from the relevant classification model,
     i.e. the latest trained model for the category specified in the request.
+    As finding and returning _all_ the positively predicted elements is expensive, this endpoint only finds as many
+    elements as is required by the request.
 
     :param workspace_id:
     :request_arg category_id:
@@ -486,22 +507,20 @@ def get_positive_predictions(workspace_id):
     all_ready_iterations = curr_app.orchestrator_api.get_all_iterations_by_status(workspace_id, category_id,
                                                                                   IterationStatus.READY)
     if len(all_ready_iterations) == 0:
-        return jsonify({'hit_count': 0, "positive_fraction": None, 'elements': []})
+        return jsonify({'elements': [], 'positive_fraction': None, 'total_count': None})
     else:
-        # Where labels are applied to duplicate texts (the default behavior), we do not want duplicates to appear in
-        # the positive predictions list
-        remove_duplicates = curr_app.config["CONFIGURATION"].apply_labels_to_duplicate_texts
-        positive_predicted_elements = curr_app.orchestrator_api.sample_elements_by_prediction(
-            workspace_id, category_id, unlabeled_only=False, required_label=LABEL_POSITIVE,
-            remove_duplicates=remove_duplicates)
+        positive_predicted_elements = curr_app.orchestrator_api.get_elements_by_prediction(
+            workspace_id, category_id, LABEL_POSITIVE, sample_size=size, start_idx=start_idx, shuffle=False,
+            remove_duplicates=False)  # For better performance in large datasets, we do not remove duplicates
         iteration, _ = all_ready_iterations[-1]
 
-        sorted_elements = sorted(positive_predicted_elements, key=lambda te: get_natural_sort_key(te.uri))
-        elements_transformed = elements_back_to_front(workspace_id, sorted_elements, category_id)
-        elements_transformed = elements_transformed[start_idx: start_idx + size]
+        elements_transformed = elements_back_to_front(workspace_id, positive_predicted_elements, category_id)
 
-        res = {'hit_count': len(positive_predicted_elements), "positive_fraction":
-               iteration.iteration_statistics["positive_fraction"], 'elements': elements_transformed}
+        positive_fraction = iteration.iteration_statistics["positive_fraction"]
+        total_positive_count = iteration.iteration_statistics.get(
+            'total_positive_count', int(positive_fraction * curr_app.orchestrator_api.get_text_element_count(workspace_id)))
+        res = {'elements': elements_transformed,
+               'positive_fraction': positive_fraction, 'total_count': total_positive_count}
         return jsonify(res)
 
 
@@ -673,9 +692,9 @@ def get_all_labeled_elements(workspace_id, category_id, label=None, size: int =
     if label is not None:
         elements = [element for element in elements
                     if element.category_to_label[category_id].label == label]
+    hit_count = len(elements)
+    elements = elements[start_idx: start_idx + size]
     elements_transformed = elements_back_to_front(workspace_id, elements, category_id)
-    hit_count = len(elements_transformed)
-    elements_transformed = elements_transformed[start_idx: start_idx + size]
     return elements_transformed, hit_count
 
 
@@ -965,9 +984,9 @@ def get_suspicious_elements(workspace_id):
     
     try:
         suspicious_elements = curr_app.orchestrator_api.get_suspicious_elements_report(workspace_id, category_id)
+        hit_count = len(suspicious_elements)
+        suspicious_elements = suspicious_elements[start_idx: start_idx + size]
         elements_transformed = elements_back_to_front(workspace_id, suspicious_elements, category_id)
-        hit_count = len(elements_transformed)
-        elements_transformed = elements_transformed[start_idx: start_idx + size]
         res = {'elements': elements_transformed, 'hit_count': hit_count}
         return jsonify(res)
     except Exception:
@@ -1037,9 +1056,9 @@ def get_elements_for_precision_evaluation(workspace_id):
     size = curr_app.config["CONFIGURATION"].precision_evaluation_size
     category_id = int(request.args['category_id'])
     random_state = len(curr_app.orchestrator_api.get_all_iterations_for_category(workspace_id, category_id))
-    positive_predicted_elements = curr_app.orchestrator_api. \
-        sample_elements_by_prediction(workspace_id, category_id, size, unlabeled_only=False,
-                                      required_label=LABEL_POSITIVE, remove_duplicates=False, random_state=random_state)
+    positive_predicted_elements = curr_app.orchestrator_api.\
+        get_elements_by_prediction(workspace_id, category_id, required_prediction=LABEL_POSITIVE, sample_size=size,
+                                   remove_duplicates=False, shuffle=True, random_state=random_state)
     elements_transformed = elements_back_to_front(workspace_id, positive_predicted_elements, category_id)
     logging.info(f"sampled {len(elements_transformed)} elements for evaluation")
     res = {'elements': elements_transformed}
@@ -1102,6 +1121,7 @@ def cancel_precision_evaluation(workspace_id):
     return jsonify(res)
 
 
+
 """
 Information on enriched tokens in a specific subset of elements, used for visualization.
 """
@@ -1155,12 +1175,11 @@ def get_predictions_enriched_tokens(workspace_id):
         res['info_gain'] = []
         return jsonify(res)
 
-    true_elements = curr_app.orchestrator_api.sample_elements_by_prediction(workspace_id, category_id, 1000,
-                                                                            unlabeled_only=True,
-                                                                            required_label=LABEL_POSITIVE)
-    false_elements = curr_app.orchestrator_api.sample_elements_by_prediction(workspace_id, category_id, 1000,
-                                                                             unlabeled_only=True,
-                                                                             required_label=LABEL_NEGATIVE)
+    true_elements = curr_app.orchestrator_api.get_elements_by_prediction(
+        workspace_id, category_id, LABEL_POSITIVE, sample_size=1000, shuffle=True, remove_duplicates=False)
+    false_elements = curr_app.orchestrator_api.get_elements_by_prediction(
+        workspace_id, category_id, LABEL_NEGATIVE, sample_size=1000, shuffle=True, remove_duplicates=False)
+
     elements = true_elements + false_elements
     boolean_labels = [LABEL_POSITIVE] * len(true_elements) + [LABEL_NEGATIVE] * len(false_elements)
 
@@ -1182,4 +1201,32 @@ def get_feature_flags():
     }
     logging.debug(f'Feature flags are: {res}')
     return jsonify(res) 
-    
\ No newline at end of file
+    
+
+@main_blueprint.route("/version", methods=['GET'])
+def get_git_describe():
+    """Retrieves the system version 
+
+    Git version is given priority, thus if git is installed 
+    in the system and a .git folder is present, the output
+    of 'git describe' is returned. If that fails, the package
+    version is returned. If that fails too, the string 'not
+    available' is returned instead.
+
+    Returns:
+        an object with version and source fields
+    """
+    try: 
+        repo = git.repo.Repo(os.path.abspath(os.path.join(__file__, os.pardir, os.pardir)))
+        version = repo.git.describe(tags=True)
+        source = 'git'
+    except:
+        try:
+            version = pkg_resources.require("label-sleuth")[0].version
+            source = 'pypi'
+        except:
+            version = 'Not available'
+            source = None
+            logging.warning(f'Could not get Label Sleuth version information')
+
+    return {'version': version, 'source': source}
\ No newline at end of file
