@@ -15,12 +15,15 @@
 
 import ast
 import dataclasses
-import jsonpickle
 import logging
 import os
+import random
 import shutil
 import sys
 import threading
+import time
+
+import jsonpickle
 import ujson as json
 import pandas as pd
 
@@ -80,7 +83,7 @@ class FileBasedDataAccess(DataAccessApi):
             intersection = doc_ids.intersection(set(self.get_all_document_uris(dataset_name)))
             if len(intersection) > 0:
                 raise AlreadyExistsException(f"{len(intersection)} documents are already in dataset '{dataset_name}'. "
-                                             f"uris: ({list(intersection)[:5]}{'...' if len(intersection)>5 else ''})",
+                                             f"uris: ({list(intersection)[:5]}{'...' if len(intersection) > 5 else ''})",
                                              list(intersection))
 
         for doc in documents:
@@ -176,6 +179,7 @@ class FileBasedDataAccess(DataAccessApi):
         :return: a List of Document objects, from the given dataset_name, matching the uris provided, containing label
         information for the TextElements of these Documents, if available.
         """
+
         def load_doc(uri):
             filename = utils.uri_to_filename(uri)
             file_path = os.path.join(doc_dump_dir, filename)
@@ -217,6 +221,13 @@ class FileBasedDataAccess(DataAccessApi):
         """
         return list(self._get_ds_in_memory(dataset_name)['uri'].values)
 
+    def get_text_element_count(self, dataset_name: str) -> int:
+        """
+        Return the total number of TextElements in the given dataset_name.
+        :param dataset_name:
+        """
+        return len(self.get_all_text_elements_uris(dataset_name))
+
     def get_all_text_elements(self, dataset_name: str) -> List[TextElement]:
         """
         Return a List of all TextElement in the given dataset_name.
@@ -385,6 +396,31 @@ class FileBasedDataAccess(DataAccessApi):
 
         return text_elements
 
+    def get_text_element_iterator(self, workspace_id, dataset_name, shuffle=False, random_state: int = 0,
+                                  remove_duplicates=False) -> Iterable[TextElement]:
+        """
+        Enables iterating over TextElement objects from the given *dataset_name*. Particularly useful where the total
+        number of elements required is not known in advance.
+        :param workspace_id:
+        :param dataset_name:
+        :param shuffle: if True, the iterator yields the text elements in random order.
+        :param random_state: provide an int seed to define a random state. Default is zero.
+        :param remove_duplicates: if True, do not include elements that are duplicates of each other.
+        """
+        corpus_df = self._get_ds_in_memory(dataset_name)
+        if remove_duplicates:
+            corpus_df = corpus_df.drop_duplicates(subset=['text'])
+        all_uris = list(corpus_df['uri'].values)
+        if shuffle:
+            random.Random(random_state).shuffle(all_uris)
+
+        # extracting one element at a time from a large dataframe can be expensive, so we fetch them in batches
+        # but yield them one by one
+        batch_size = 1000
+        for i in range(0, len(all_uris), batch_size):
+            batch_uris = all_uris[i:i + batch_size]
+            yield from self.get_text_elements_by_uris(workspace_id, dataset_name, batch_uris)
+
     def get_all_dataset_names(self) -> List[str]:
         """
         :return: a list of all available datset names
@@ -414,12 +450,12 @@ class FileBasedDataAccess(DataAccessApi):
                 if self._dataset_exists(dataset_name):
                     logging.info(f"reading dataset '{dataset_name}' csv file")
                     dataset_file_path = self._get_dataset_dump_filename(dataset_name)
-                    df = pd.read_csv(dataset_file_path)
-                    logging.info(f"csv file for dataset '{dataset_name}' read successfully")
-                    # convert value of TextElement fields to their proper formats
-                    df = df.where(pd.notnull(df), None)
-                    for field in ['span', 'metadata']:
-                        df[field] = [ast.literal_eval(x) if x is not None else {} for x in df[field].values]
+
+                    df = pd.read_csv(dataset_file_path, converters=
+                            {"span": lambda span: [tuple(int(x) for x in span[2:-2].split(','))],
+                             "metadata": lambda metadata: ast.literal_eval(metadata) if metadata != '{}' else {}})
+
+                    logging.info(f"csv file for dataset '{dataset_name}' read and processed successfully")
                 else:
                     raise Exception(f'Dataset "{dataset_name}" does not exist.')
                 self.ds_in_memory[dataset_name] = df
@@ -518,7 +554,7 @@ class FileBasedDataAccess(DataAccessApi):
         """
         unique_to_id = {}
         df['text_unique_id'] = df['text'].apply(
-            lambda x: len(unique_to_id)-1 if x not in unique_to_id and not unique_to_id.update({x: len(unique_to_id)})
+            lambda x: len(unique_to_id) - 1 if x not in unique_to_id and not unique_to_id.update({x: len(unique_to_id)})
             else unique_to_id[x])
         return df
 
@@ -550,3 +586,6 @@ class FileBasedDataAccess(DataAccessApi):
 
     def _get_workspace_labels_dir(self, workspace_id):
         return os.path.join(self.output_dir, 'user_labels', str(workspace_id))
+
+    def preload_dataset(self, dataset_name):
+        self._get_ds_in_memory(dataset_name)
