@@ -45,9 +45,10 @@ def filename_to_uri(filename):
 def build_text_elements_from_dataframe_and_labels(df, labels_dict):
     # text element fields are extracted from the dataframe, with the exception of the labels, which are stored elsewhere
     element_data_columns = TextElement.get_field_names() - {'category_to_label'}
-    element_dicts = df[element_data_columns].to_dict('records')
+    element_dicts = map(lambda row: dict(zip(element_data_columns, row)), df[element_data_columns].values)
     text_elements = [TextElement(**d, category_to_label=labels_dict.get(d['uri'], {}).copy())
                      for d in element_dicts]
+
     return text_elements
 
 
